#include "query.h"
#include "utils.h"
#include "i18n.h"

#include <unity/scopes/CategorisedResult.h>
#include <unity/scopes/CategoryRenderer.h>
#include <unity/scopes/QueryBase.h>
#include <unity/scopes/SearchReply.h>
#include <unity/scopes/Department.h>
#include <unity/scopes/CannedQuery.h>

#include <QDebug>
#include <QJsonArray>
#include <QJsonDocument>
#include <QJsonObject>
#include <QNetworkAccessManager>
#include <QNetworkReply>
#include <QNetworkRequest>
#include <QUrl>
#include <QStringList>
#include <QLocale>
#include <QDate>

#include <mediascanner/MediaFile.hh>
#include <mediascanner/MediaStore.hh>
#include <mediascanner/Filter.hh>
#include <mediascanner/scannercore.hh>

#include <locale.h>

#define MAX_RESULTS 36

namespace us = unity::scopes;
static const char RENDERER[] = R"(
{
  "schema-version": 1,
  "template": {
    "category-layout": "grid",
    "card-size": "small"
  },
  "components": {
    "art":  "art"
  }
}
)";


Query::Query(const us::CannedQuery &query, const us::SearchMetadata & metadata, Scope &scope) :
    SearchQueryBase(query, metadata),
    cannedQuery(query),
    scope_(scope),
    query_(query.query_string())
{

    textdomain(GETTEXT_DOMAIN.toStdString().c_str());
    const QString scopePath = QString::fromStdString(scope_.scope_directory());
    QString tdir = QString("%1/locale/").arg(scopePath);
    QString basedir = QString(bindtextdomain(GETTEXT_DOMAIN.toStdString().c_str(), tdir.toStdString().c_str()));
    setlocale(LC_ALL,"");
    locale = QLocale();

    //use mediascanner Filter to set order (used in query() below) to
    //newest->oldest
    filter = new mediascanner::Filter();
    filter->setOrder(mediascanner::MediaOrder::Date);
    filter->setReverse(true);
}

Query::~Query()
{
}

void Query::cancelled()
{
}

const QString getDate(QLocale &loc, const std::string &date_in)
{
    QString date_ = qstr(date_in);
    QStringList date_items_in = date_.split("T");
    QStringList date_items = date_items_in[0].split("-");
    int year = date_items[0].toInt();
    int month = date_items[1].toInt();
    int day = date_items[2].toInt();
    QDate date = QDate(year, month, day);
    QString date_l = loc.toString(date);
    return date_l;
}

QString getTime(QLocale &loc, std::string date)
{
    QString date_ = qstr(date);
    QStringList time_l = date_.split("T");
    QStringList time_items = time_l[1].split(":");
    QTime time = QTime(time_items[0].toInt(), time_items[1].toInt(), time_items[2].toInt());
    return loc.toString(time);
}

void Query::run(us::SearchReplyProxy const& reply)
{
    us::CategoryRenderer renderer(RENDERER);

    auto cat1 = reply->register_category("today", _("Today"), "", renderer);
    auto cat7 = reply->register_category("previous7", _("Previous 7 days"), "", renderer);
    auto cat30 = reply->register_category("previous30", _("Previous 30 days"), "", renderer);
    auto catOlder = reply->register_category("older", _("Older"), "", renderer);

    QDate searchDate;
    bool searching = false;
    // handle search:
    if (query_ != "")
    {
        searching = true;
        QString qry = qstr(query_);
        QStringList parts = qry.split(" ");
        int dy = 0;
        int mo = 0;
        int yr = 0;
        if (parts.size() > 0)
        {
            dy = parts[0].toInt();
            if (parts.size() > 1)
                mo = parts[1].toInt();
            if (parts.size() > 2)
                yr = parts[2].toInt();
        }
        QDate date = QDate::currentDate();
        if (mo == 0 )
            mo = date.month();
        if (yr < 2000)
            yr = date.year();
        searchDate = QDate(yr, mo, dy);
    }

    const std::string noquery = "";

    std::list<mediaUnknown> resultsUnknown;

    for (const auto &media : scope_.store->query(noquery, mediascanner::MediaType::ImageMedia, (*filter))) {
        QDate mDate;
        QDateTime mDateTime_local;
        bool unknownDate = false;
        Query::mediaUnknown unknown;
        const QString media_date = qstr(media.getDate());
        qDebug() << QString("======== media_date: %1").arg(media_date);
        if (media_date.isEmpty())
        {
            unknown.date = _("Unknown");
            unknown.time = _("Unknown");
            unknown.artUri = media.getUri();
            QString mediaUri = QString::fromStdString(media.getUri());
            mediaUri.replace("file://", "photo://");
            unknown.uri = mediaUri.toStdString();
            unknown.title = media.getTitle();
            qDebug() << "My Photos. This photo has no timestamp" << QString::fromStdString(media.getUri());
            unknownDate = true;
            resultsUnknown.emplace_back(unknown);
        } else {
            qDebug() << "==== MS url: " << QString::fromStdString(media.getUri());
            QDateTime mDateTime_utc = QDateTime::fromString(media_date, Qt::ISODate);
            mDateTime_utc.setTimeSpec(Qt::UTC);
            qDebug() << "==== media_date: " << media_date;
            qDebug() << "==== mDataTime_utc: " << mDateTime_utc.toString();
            mDateTime_local = mDateTime_utc.toLocalTime();
            qDebug() << "==== mDataTime_local: " << mDateTime_local.toString();
            mDate = mDateTime_local.date();
        }
        qDebug() << "==== mDate: " << mDate.toString();
        if ((searching) && (mDate != searchDate))
            continue;

        QDate today = QDate::currentDate();
        QDate sevenDays = QDate::currentDate().addDays(-7);
        QDate thirtyDays = QDate::currentDate().addDays(-37);
        qDebug() << "==== today: " << today;
        qDebug() << "==== 7 days ago: "<< sevenDays;
        qDebug() << "==== 37 days ago: "<< thirtyDays;

        us::CategorisedResult res(cat1);
        if (unknownDate) {
            continue;
        } else if (mDate == today) {
            res.set_category(cat1);
        } else if (mDate >= sevenDays) {
            res.set_category(cat7);
        } else if (mDate >= thirtyDays) {
            res.set_category(cat30);
        } else {
            res.set_category(catOlder);
        }

        QString title = qstr(media.getTitle());
        QString mediaUri = QString::fromStdString(media.getUri());
        mediaUri.replace("file://", "photo://");
        res.set_uri(mediaUri.toStdString());
        res.set_dnd_uri(media.getUri());
        res.set_art(media.getUri());
        res.set_title(title.toStdString());
        std::string date = _("Date");
        if (!unknownDate)
        {
            QDate d = mDateTime_local.date();
            QTime t = mDateTime_local.time();
            res["date"] = "<strong>" + date + "</strong>: " + d.toString(locale.dateFormat()).toStdString();
            std::string time = _("Time");
            res["time"] = "<strong>" + time + "</strong>: " + t.toString(locale.timeFormat()).toStdString();
        }
        if(!reply->push(res))
        {
            return;
        }
    }
    if (resultsUnknown.size() > 0)
    {
        auto catUnknown = reply->register_category("unknown", _("Unknown"), "", renderer);
        for (mediaUnknown unknown : resultsUnknown)
        {
            us::CategorisedResult res(catUnknown);
            res.set_uri(unknown.uri);
            res.set_art(unknown.artUri);
            res.set_title(unknown.title);
            if(!reply->push(res))
            {
                return;
            }
        }
    }
}

